require 'spec_helper'

describe Validocno::Validator do
  let(:doc_number) { "A1234567" }
  let(:validator) { Validocno::Validator.new(doc_number) }
  let(:invalid_post_validator) { Validocno::Validator.new("00000000") }

  let(:valid_nr_response) do
    body = File.new('spec/fixtures/post_response.html').read
    { :body => body, :document_number => doc_number }
  end

  let(:expired_nr_response) do
    body = File.new('spec/fixtures/post_response.html').read
    body = body.gsub(doc_number, "A0111856")
    body = body.gsub("kehtiv", "kehtetu")
    { :body => body, :document_number => "A0111856" }
  end

  let(:invalid_nr_response) do
    body = File.new('spec/fixtures/post_response.html').read
    body = body.gsub(doc_number, "A1234567890")
    body = body.gsub(
      "Dokument A1234567890 on kehtiv.",
      "Dokumenti A1234567890 ei ole välja antud."
    )
    { :body => body, :document_number => "A1234567890" }
  end

  let(:invalid_post_response) do
    body = File.new('spec/fixtures/invalid_post_response.html').read
    { :body => body, :document_number => "00000000" }
  end

  describe '.new' do
    it 'initializes with a document number' do
      expect(validator).to be
      expect { validator }.to_not raise_error
    end

    it 'raises an error without a document number' do
      expect { Validocno::Validator.new }.to raise_error(ArgumentError)
    end
  end

  describe '#validate' do
    let(:invalid_doc_number) { "A1234567890" }
    let(:expired_doc_number) { "A0111856" }
    let(:validator_with_invalid) do
      Validocno::Validator.new(invalid_doc_number)
    end

    let(:validator_with_expired) do
      Validocno::Validator.new(expired_doc_number)
    end

    it 'returns hash with :valid and :message keys' do
      mock_response_with :valid_nr
      expect(validator.validate).to have_key(:valid)
      expect(validator.validate).to have_key(:message)
    end

    it 'validates document with valid number' do
      mock_response_with :valid_nr

      expect(validator.validate).to include(
        :valid => true,
        :message => "Dokument #{doc_number} on kehtiv."
      )
    end

    it 'validates expired document' do
      mock_response_with :expired_nr
      expect(validator_with_expired.validate).to include(
        :valid => false,
        :message => "Dokument #{expired_doc_number} on kehtetu."
      )
    end

    it 'validates document with invalid number' do
      mock_response_with :invalid_nr
      expect(validator_with_invalid.validate).to include(
        :valid => false,
        :message => "Dokumenti #{invalid_doc_number} ei ole välja antud."
      )
    end

    it 'fails if node not found' do
      mock_response_with :invalid_post
      expect { invalid_post_validator.validate }.to raise_error(StandardError)
    end
  end

  private

  def mock_response_with(response)
    response_result = send("#{response}_response")

    stub_request(:get, "https://www.politsei.ee/qr/?qr=#{response_result[:document_number]}").
      with(:headers => {
        "Accept" => "*/*",
        "Accept-Encoding" => "gzip, deflate",
        "Host" => "www.politsei.ee",
        "User-Agent" => "rest-client/2.0.2 (darwin17.2.0 x86_64) ruby/2.1.10p492"
      }).to_return(
        :status => 200,
        :body => response_result[:body],
        :headers => {}
      )
  end
end
